@@ -13,24 +13,24 @@ namespace c10 {
 /**
  * convert ScalarType enum values to TypeMeta handles
  */
-static inline caffe2::TypeMeta scalarTypeToTypeMeta(ScalarType scalar_type) {
+inline caffe2::TypeMeta scalarTypeToTypeMeta(ScalarType scalar_type) {
   return caffe2::TypeMeta::fromScalarType(scalar_type);
 }
 
 /**
  * convert TypeMeta handles to ScalarType enum values
  */
-static inline ScalarType typeMetaToScalarType(caffe2::TypeMeta dtype) {
+inline ScalarType typeMetaToScalarType(caffe2::TypeMeta dtype) {
   return dtype.toScalarType();
 }
 
 /**
  * typeMetaToScalarType(), lifted to optional
  */
-static inline optional<at::ScalarType> optTypeMetaToScalarType(
-    optional<caffe2::TypeMeta> type_meta) {
+inline std::optional<at::ScalarType> optTypeMetaToScalarType(
+    std::optional<caffe2::TypeMeta> type_meta) {
   if (!type_meta.has_value()) {
-    return c10::nullopt;
+    return std::nullopt;
   }
   return type_meta->toScalarType();
 }
@@ -38,19 +38,19 @@ static inline optional<at::ScalarType> optTypeMetaToScalarType(
 /**
  * convenience: equality across TypeMeta/ScalarType conversion
  */
-static inline bool operator==(ScalarType t, caffe2::TypeMeta m) {
+inline bool operator==(ScalarType t, caffe2::TypeMeta m) {
   return m.isScalarType(t);
 }
 
-static inline bool operator==(caffe2::TypeMeta m, ScalarType t) {
+inline bool operator==(caffe2::TypeMeta m, ScalarType t) {
   return t == m;
 }
 
-static inline bool operator!=(ScalarType t, caffe2::TypeMeta m) {
+inline bool operator!=(ScalarType t, caffe2::TypeMeta m) {
   return !(t == m);
 }
 
-static inline bool operator!=(caffe2::TypeMeta m, ScalarType t) {
+inline bool operator!=(caffe2::TypeMeta m, ScalarType t) {
   return !(t == m);
 }
 
